#!/usr/bin/env sh
# automatically generated by auxilium

set -eE

sphinx-apidoc -o {{ aux.docs.root }} {{ aux.project.module_dir }}
sphinx-build -M html {{ aux.docs.root }} devops/cache/docs{% if aux.docs.get('strict', True) == True %} -W{% endif %}
python devops/docs/postprocess_html.py devops/cache/docs/html/{{ aux.project.second_name }}.html

if [ "$1" != "" ]
then
    open "devops/cache/docs/html/index.html"
fi
