# automatically generated by auxilium
[metadata]
version = attr: {{ aux.project.name }}.__version__

[options]
zip_safe = False
packages = {%if "." in aux.project.name %}find_namespace{%else%}find{%endif%}:
{% if aux.project.source_dir != "." %}
package_dir =
	= {{ aux.project.source_dir }}
{% endif %}
python_requires = >={{ aux.project.minimal_version }}
install_requires =

[options.entry_points]
console_scripts =

[options.package_data]
* = *.typed

[options.extras_require]
dev =

[tool:pytest]
cache_dir = devops/cache/pytest
{% for key, val in aux.get("pytest",{}).items() -%}
    {{ key }} = {{ val }}
{% endfor -%}
