version: '3.6'
{% import "macros.jinja2" as x %}
services:
    {{ opts.full_name }}:
        {{ x.include_docker_build(aux, opts) }}
        {{ x.include_platform(aux) }}
        image: {{ opts.full_name }}
        volumes:
            - {{ aux.docker.source_dir }}:/home/container/workdir:rw
        working_dir: /home/container/workdir
        command: [pytest{% if "test_dir" in opts %}, {{opts.test_dir}}{% endif %}]
