# automatically generated by auxilium
.r-docker-cache-off:
    if: $DOCKER_CACHE == "no"

.r-pipeline:
    if: $CI_PIPELINE_SOURCE == "pipeline"

.r-push-no-mr:
    if: '$CI_PIPELINE_SOURCE == "push" && $CI_OPEN_MERGE_REQUESTS == null && $CI_COMMIT_TAG == null'

.r-web:
    if: $CI_PIPELINE_SOURCE == "web"

.r-mr:
    if: $CI_PIPELINE_SOURCE == "merge_request_event"

.r-vip-mr:
    if: {% for branch in aux.gitlab.vip_branches -%}{%- if branch != aux.gitlab.vip_branches | first %} || {% endif -%}
    $CI_MERGE_REQUEST_TARGET_BRANCH_NAME == "{{ branch }}"
    {%- endfor%}

.r-vip-push:
    if: {% for branch in aux.gitlab.vip_branches -%}{%- if branch != aux.gitlab.vip_branches | first %} || {% endif -%}
    $CI_COMMIT_BRANCH == "{{ branch }}"
    {%- endfor%}

.r-develop-mr:
    if: $CI_MERGE_REQUEST_TARGET_BRANCH_NAME == "develop"

.r-develop-push:
    if: '$CI_COMMIT_BRANCH == "develop" && $CI_COMMIT_TAG == null'

.r-release-mr:
    if: $CI_MERGE_REQUEST_TARGET_BRANCH_NAME == "{{ aux.gitlab.release_branch }}"

.r-release-push:
    if: '$CI_COMMIT_BRANCH == "{{ aux.gitlab.release_branch }}" && $CI_COMMIT_TAG == null'
