#!/usr/bin/env bash
# automatically generated by auxilium

set -eE

PIP_CONFIG_FILE=devops/pip/pip.conf pip install -e "$PWD[test,dev{% if "docs" in aux %},docs{%endif%}]" --config-settings editable_mode=strict
pre-commit install --config=devops/pre-commit/config.yaml
